% % Koya University - Safar Maghdid Asaad 
%
% Copyright (c) 2022, Koya University and Erbil Polytechnic University - Safar Maghdid Asaad 
% *************************************************
% -------------------------------------------------
% FILE       : Augmentation.m
% DESCRIPTION: A function to perform the SMATE augmentation on the original
%              RSSI dataset

%              based on work of "Safar Mahdid Asaad"
%
% -------------------------------------------------
% *************************************************
% Mean and Standard Deviation Based Augmentation TEchnique (MSATE) is 
% proposed to enhance the quantity of RSSI samples for each class or RP. 
% In data analysis, one of the methods for increasing the quantity of 
% RSSI data samples is known as "data augmentation.". 
% When the data augmentation involves the creation of freshly produced
% synthetic RSSI data that is based on previously gathered datasets or
% the addition of slightly modified copies of RSSI data that already 
% exists. It also serves as a regularizer and aids in reducing 
% overfitting while training a Deep Learning model. 
% The suggested approach, MSATE, produces a new N number of RSSI values 
% for each WAP at a specified position (p) based on the mean and 
% standard deviation computation for the pre-collected RSSI values at 
% point (p). There will therefore be N new RSSI records at (p).
% The mathematical calculations on which the implementations were based 
% are further explained in the accompanying paper.
    % T_sorted_Train is a 1x1 cell that contains X Y with 14 RSSI features for
    % 14 APs
    % N is the number of mimic (imitate) records.
    % R is the number of original records for each reference point (RP) or
    % (P)
function [new_data_set] = Augmentation(T_sorted_Train, N, R)

    c=1;
    for i = 1:R:length(T_sorted_Train{1}.(1+2))
        for j = 1:14
          meansWAPs(c,j)=mean(T_sorted_Train{1}.(j+2)(i:i+(R-1)));
          stdWAPs(c,j)=std(T_sorted_Train{1}.(j+2)(i:i+(R-1)));
        end
        c = c+1;
    end
    % calculate mean of the std of all the points
    mean_stdWAPs = mean(stdWAPs);
    % generate N random numbers between a and b
    a = 2; b = -2;
%     N = 15 
    temp = [];
    for i = 1: 14
        r = a + (b-a).*rand(N,1);
        temp = [temp r];
    end
    r = temp;
    % add each of the r numbers to each of the mean_stdWAPs
    N_mean_stdWAPs = r + mean_stdWAPs; 
    
    % Adding the generated N_mean_stdWAPs to the mean of each the reference
    % points to generate new N records.
    c=1;
    for i = 1: length(meansWAPs)
        new_N_recors_At_eachPosition(c:c+14, :) = N_mean_stdWAPs + meansWAPs(i, :);
        c = c + N;
    end
    
    % Add the new N records to the original 10 records at each reference point
    c = 1;
    c2=1;
    for i = 1:R:length(T_sorted_Train{1}.(1+2))
        X = ones(N, 1);
        X = X * T_sorted_Train{1}.(1)(i, 1);
        Y = ones(N, 1);
        Y = Y * T_sorted_Train{1}.(2)(i, 1);
        new_N_recors_At_eachPosition_(c:c+(N-1), :) = [X Y new_N_recors_At_eachPosition(c:c+(N-1), :)];
        array_data = table2array(T_sorted_Train{1}(i:i+(R-1), :));
        new_data_set(c2:(c2+N+(R-1)), :) = [array_data; new_N_recors_At_eachPosition_(c:c+(N-1), :)];
        c = c+N;
        c2 = c2+N+R;
    end



end