% % Koya University - Safar Maghdid Asaad 
%
% Copyright (c) 2022, Koya University and Erbil Polytechnic University - Safar Maghdid Asaad 
% *************************************************
% -------------------------------------------------
% FILE       : MSATE_Norm_LSTM.m
% DESCRIPTION: A source code to implement the proposed 
%              SMATE_Norm_LSTM solution

%              based on work of "Safar Mahdid Asaad"
%
% -------------------------------------------------
% *************************************************
clear all
close all
clc
for i=1:1
    T1=readtable(sprintf('Original.csv'));
    sortedNames_WAPs = T1.Properties.VariableNames(3:end);
    T_sorted_Train{i} = [T1(:,1:2) T1(:,sortedNames_WAPs)];
end


% Mean and Standard Deviation Based Augmentation TEchnique (MSATE) is 
% proposed to enhance the quantity of RSSI samples for each class or RP. 
% In data analysis, one of the methods for increasing the quantity of 
% RSSI data samples is known as "data augmentation.". 
% When the data augmentation involves the creation of freshly produced
% synthetic RSSI data that is based on previously gathered datasets or
% the addition of slightly modified copies of RSSI data that already 
% exists. It also serves as a regularizer and aids in reducing 
% overfitting while training a Deep Learning model. 
% The suggested approach, MSATE, produces a new N number of RSSI values 
% for each WAP at a specified position (p) based on the mean and 
% standard deviation computation for the pre-collected RSSI values at 
% point (p). There will therefore be N new RSSI records at (p).
% The mathematical calculations on which the implementations were based 
% are further explained in the accompanying paper.

% Related to Augmentation
% T_sorted_Train is a 1x1 cell that contains X Y with 14 RSSI features for
% 14 APs
% N is the number of mimic (imitate) (Copy) (Generate) records, here it is 15
% R is the number of original records for each reference point (RP) here it
% is 10
[new_data_set] = Augmentation(T_sorted_Train, 15, 10); %MSATE
% csvwrite('augmented.csv', new_data_set);

% The proposed solution applies the normalization process over the 
% expanded dataset. 
% Normalization is a method that is frequently used as a preprocessing 
% mechanism in machine learning data preparation. The purpose of 
% normalization is to maintain the RSSI values' ranges while converting 
% the values of a dataset's numeric RSSI columns to a comparable scale. 
% Not every dataset has to be normalized for machine learning. Only when 
% features have separate ranges is it essential. In this work, 
% the normalization is done in order to reduce the fluctuation problems,
% particularly those that arise when using a variety of platforms and 
% devices to improve the performance of the LSTM.
%% normaliz the datasets
[new_data_set] = Normalization(new_data_set);
% csvwrite('normalized.csv', new_data_set);

% save new_data_set_Aug_Norm new_data_set

% load new_data_set_Aug_Norm.mat;

%% plot indoor map points
X_Train=new_data_set(:, 1);
Y_Train=new_data_set(:, 2);
% figure(1)
% plot(X_Train,Y_Train,'-ok');
% % Plotting WAPs RSSI values
% figure(2)
% colors{1}=['*k'];
% colors{2}=['*g']; 
% colors{3}=['*r']; 
% colors{4}=['*b'];
% colors{5}=['*m'];
% colors{6}=['*c'];
% colors{7}=['*y'];
% colors{8}=['ok'];
% colors{9}=['og'];
% legendCell=[];

c=1;
for i=1:25:length(new_data_set(:, 1))
   Y_train(i:i+24)=c;
   c=c+1;
end
Y_train=Y_train';

c=1;
for j=1:1
    RSS_Train=[]; RSS_Test=[];
    for i=1:14
       RSS_Train=[RSS_Train new_data_set(:, i+2)];
    end
    RSS_Train=[new_data_set(:, 1) new_data_set(:, 2) RSS_Train Y_train];
    temp=[]; new_y=[];
    for i=1:length(RSS_Train)
        if(mod( i , 5 ) == 0 )
            RSS_Test=[RSS_Test; RSS_Train(i,:)];
        else
            temp=[temp; RSS_Train(i,:)];
        end
    end

    RSS_Train=temp; %temp2
    X=RSS_Train(:,3:end-1);
    c=1;
    for i=1:length(X)
        Xtr{c}=X(i,:)';
        c=c+1;
    end
    Xtr=Xtr';
    XTrain=Xtr;
    Y_train=RSS_Train(:,end);
    YTrain=categorical(Y_train);
    XT=RSS_Test(:,3:end-1);
    YTest=RSS_Test(:,end);
    c=1;
    for i=1:length(XT)
        XTest{c}=XT(i,:)';
        c=c+1;
    end
    XTest=XTest';



%     %% LSTM algorithm

    featureDimension = 14;            
    miniBatchSize = 64;
    numClasses = numel(unique(YTrain));
    LSTMDepth = 1;
    numHiddenUnits = 300

layers = sequenceInputLayer(featureDimension);

for i = 1:LSTMDepth
    layers = [layers;lstmLayer(numHiddenUnits,OutputMode="sequence")];
    dropoutLayer(0.6)
end
layers = [layers;lstmLayer(numHiddenUnits,OutputMode="last")];
layers = [layers
                dropoutLayer(0.6)
                fullyConnectedLayer(numClasses, "Name","fc")
                softmaxLayer("Name","softmax")
                classificationLayer("Name","classification")];




    validationFrequency = floor(numel(YTrain)/miniBatchSize);

    options = trainingOptions('adam', ...
    InitialLearnRate= 0.001, ...
    MaxEpochs= 140, ...
    LearnRateSchedule="piecewise", ...
    LearnRateDropPeriod= 8, ...
    LearnRateDropFactor=0.22, ...
    MiniBatchSize=miniBatchSize, ...
    Shuffle="every-epoch", ...
    Verbose=false, ...
    Plot ='training-progress', ...
    ValidationData={XTest,categorical(YTest)}, ...
    ValidationFrequency=validationFrequency);



    %train rows labels instead of positions
    
    net = trainNetwork(XTrain,YTrain,layers,options);
    %predict the labels
    YPred = classify(net,XTest);
    acc_LSTM = sum(YPred == categorical(YTest))./numel(categorical(YTest))
    for i=1:length(YPred)
       yp=YPred(i);
       X_predicted(i)=RSS_Test(double(yp)*5,1);
       Y_predicted(i)=RSS_Test(double(yp)*5,2);
       X_true(i)=RSS_Test(i,1);
       Y_true(i)=RSS_Test(i,2);
    end
    error=[];
    for i=1:length(YPred)
        error(i)=sqrt( ((X_predicted(i)-X_true(i))^2+(Y_predicted(i)-Y_true(i))^2)/2);
    end

    plot(1:length(double(YPred)), double(YPred), 1:length(double(YTest)), double(YTest));
    legend('Output', 'Target');


    average_error=mean(error)

    figure(3)
    cdfplot( error ) 
    xlabel('Estimated Positioning Error (meters)')
    ylabel('CDF')
     hold on

%



end
