% % Koya University - Safar Maghdid Asaad 
%
% Copyright (c) 2022, Koya University and Erbil Polytechnic University - Safar Maghdid Asaad 
% *************************************************
% -------------------------------------------------
% FILE       : Normalization.m
% DESCRIPTION: A function to Normalize the RSSI data samples
%              based on work of "Safar Mahdid Asaad"
%
% -------------------------------------------------
% *************************************************
% The proposed solution applies the normalization process over the 
% expanded dataset. 
% Normalization is a method that is frequently used as a preprocessing 
% mechanism in machine learning data preparation. The purpose of 
% normalization is to maintain the RSSI values' ranges while converting 
% the values of a dataset's numeric RSSI columns to a comparable scale. 
% Not every dataset has to be normalized for machine learning. Only when 
% features have separate ranges is it essential. In this work, 
% the normalization is done in order to reduce the fluctuation problems,
% particularly those that arise when using a variety of platforms and 
% devices to improve the performance of the LSTM.
% The mathematical calculations on which the implementations were based 
% are further explained in the accompanying paper.
function [new_data_set] = Normalization(new_data_set)


    %% normaliz the datasets
    c=1;
for j=1:14
  meansWAPs_all(1,j)=mean(new_data_set(:, j+2));
  stdWAPs_all(1,j)=std(new_data_set(:,j+2));
end
for j=1:14
    for h=1:length(new_data_set(:, j+2))
      new_data_set(h, j+2)=(new_data_set(h, j+2)-meansWAPs_all(1,j))/stdWAPs_all(1,j);
    end
end
end